import { existsSync } from 'node:fs';
import { createRequire } from 'module';
import { resolve } from 'node:path';
import { normalizePath } from 'zuby/utils/pathUtils.js';
export async function startTunnel(opts) {
    const require = createRequire(import.meta.url);
    const untunDistPath = normalizePath(require.resolve('untun'));
    const cloudflaredPath = normalizePath(resolve(untunDistPath, '..', 'chunks', 'index.mjs'));
    const { installCloudflared, startCloudflaredTunnel, cloudflaredBinPath } = await import(`file:///${cloudflaredPath}`);
    const url = `${opts.protocol || 'http'}://${opts.hostname ?? 'localhost'}:${opts.port ?? 3000}`;
    if (!existsSync(cloudflaredBinPath)) {
        await installCloudflared();
    }
    const args = [['--url', url], opts.verifyTLS ? undefined : ['--no-tls-verify', '']].filter(Boolean);
    const tunnel = await startCloudflaredTunnel(Object.fromEntries(args));
    const cleanup = async () => {
        await tunnel.stop();
    };
    for (const signal of ['SIGINT', 'SIGUSR1', 'SIGUSR2']) {
        process.once(signal, cleanup);
    }
    return {
        getURL: async () => await tunnel.url,
        close: async () => {
            await cleanup();
        },
    };
}
